%LET lo = ./LibreOffice;
FILENAME lofile ZIP "&lo./lo_sample.ods";
FILENAME lodir "&lo./";
LIBNAME sasdata V9 "&lo./";

%LET inlength = 32760;

DATA _NULL_;
  INFILE lofile(content.xml) RECFM = N LRECL = &inlength.;
  RETAIN pos 1;
  FILE lodir(temp1.xml) RECFM = V LRECL = 2000;
  LENGTH record $&inlength.
         char1 $1
         ;
  INPUT;
  record = _INFILE_;
  reclen = LENGTH(record);
  len = 1;
  DO i = 1 TO reclen;
    char1 = SUBSTR(_INFILE_, i, 1);
    SELECT;
      WHEN (INDEX("0A0D"X, char1) > 0);
      WHEN (char1 = '>') DO;
        pos + 1;
        PUT @pos char1.;
        pos = 1;
      END;
      WHEN (char1 = '<') DO;
        PUT;
        pos = 1;
        PUT @pos char1. @;
      END;
      OTHERWISE DO;
        pos + 1;
        PUT @pos char1 @;
      END;
    END;
  END;
RUN;

DATA sasdata._xml (KEEP = _record _level _first _last _text _type);
  INFILE lodir(temp1.xml) RECFM = V LRECL = 2000;
  LENGTH _record _level _first _last 8
         _text $2000
         _type $16
         ;
  RETAIN _level 0 _type ' ';
  INPUT;
  _text = _INFILE_;
  _record = _N_;
  SELECT;
    WHEN (_text =: '<?xml') DO; _level = 0; _first = 1; END;
    WHEN (_text =: '<office:body ') DO;
      _level = 1;
      _first = 1;
      IF INDEX(REVERSE(STRIP(_text)), '>/') > 0 THEN _last = 1;
    END;
    WHEN (_text =: '</office:body') DO;
      _level = 1;
      _last = 1;
    END;
    WHEN (_text =: '<table:table ') DO;
      _level = 2;
      _first = 1;
      IF INDEX(REVERSE(STRIP(_text)), '>/') > 0 THEN _last = 1;
    END;
    WHEN (_text =: '</table:table>') DO;
      _level = 2;
      _last = 1;
    END;
    WHEN (_text =: '<table:table-row ') DO;
      _level = 3;
      _first = 1; 
      IF INDEX(REVERSE(STRIP(_text)), '>/') > 0 THEN _last = 1;
    END;
    WHEN (_text =: '</table:table-row') DO;
      _level = 3;
      _last = 1;
    END;
    WHEN (_text =: '<table:table-cell ') DO;
      _level = 4;
      _first = 1;
      IF INDEX(REVERSE(STRIP(_text)), '>/') > 0 THEN _last = 1;
      _offset = INDEX(_text, "office:value-type=");
      IF _offset > 0 THEN _type = SCAN(SUBSTR(_text, _offset +
                         LENGTH("office:value-type=") + 1), 1, """");
    END;
    WHEN (_text =: '</table:table-cell') DO;
      _level = 4;
      _last = 1;
    END;
    WHEN (_text =: '<text:p>') DO; _level = 5; _first = 1; END;
    WHEN (_text =: '</text:p>') DO; _level = 5; _last = 1; END;
    WHEN (_text =: '<office:') RETURN;
    WHEN (_text =: '</office:') RETURN;
    WHEN (_text =: '<style ') RETURN;
    WHEN (_text =: '</style') RETURN;
    WHEN (_text =: '<number ') RETURN;
    WHEN (_text =: '</number') RETURN;
    WHEN (_text =: '<table: ') RETURN;
    WHEN (_text =: '</table:') RETURN;
    WHEN (_text =: '<text') RETURN;
    WHEN (_text =: '</text') RETURN;
    WHEN (_text = ' ') RETURN;
    OTHERWISE;
  END;
  IF _text =: '<?xml' OR _level > 0 THEN OUTPUT;
  IF _level = 4 AND _last = 1 THEN _type = "";
RUN;

DATA sasdata._xmltable (KEEP = _record_first _record_last _table);
  SET sasdata._xml;
  WHERE _level = 2;
  LENGTH _table $32;
  RETAIN _record_first _record_last . _table ' ';
  IF _first = 1 THEN DO;
    _table = TRANSLATE(STRIP(SCAN(_text, 2, '"')), '_', ' ');
    _record_first = _record;
    _record_last = .;
  END;
  ELSE IF _last = 1 THEN DO;
    _record_last = _record;
    OUTPUT;
  END;
RUN;

PROC SORT DATA = sasdata._xmltable;
  BY _table;
RUN;

DATA _xmlcolumn (KEEP = _table _columns);
  SET sasdata._xml;
  WHERE (_level IN (2 4) AND _first = 1) OR _level = 3;
  LENGTH _table $32
         _columns 8
         ;
  RETAIN _table ' ' _columns .;
  IF _level = 2 THEN DO;
    _table = TRANSLATE(STRIP(SCAN(_text, 2, '"')), '_', ' ');
    RETURN;
  END;
  ELSE IF _level = 3 AND _first = 1 THEN DO;
    _columns = 0;
    RETURN;
  END;
  ELSE IF _level = 4 THEN DO;
    _columns + 1;
    RETURN;
  END;
  ELSE IF _level = 3 AND _last = 1 THEN DO;
    OUTPUT;
  END;
RUN;

PROC SUMMARY DATA = _xmlcolumn NWAY;
  CLASS _table;
  VAR _columns;
  OUTPUT OUT = sasdata._xmlcolumn (DROP = _freq_ _type_) MAX =;
RUN;

PROC SORT DATA = sasdata._xmlcolumn;
  BY _table;
RUN;

DATA cntlin (KEEP = fmtname start end label hlo type);
  LENGTH fmtname $7
         start end label $32
         hlo type $1
         ;
  MERGE sasdata._xmltable (IN = t)
        sasdata._xmlcolumn (IN = c)
        ;
  BY _table;
  hlo = ' ';
  fmtname = 'rngtab';
  type = 'N';
  start = STRIP(PUT(_record_first, 16.));
  end = STRIP(PUT(_record_last, 16.));
  label = _table;
  OUTPUT;
  fmtname = 'tabcol';
  type = 'C';
  start = _table;
  end = _table;
  label = _columns;
  OUTPUT;
RUN;

PROC SORT DATA = cntlin NODUPKEYS;
  BY fmtname start;
RUN;

PROC FORMAT CNTLIN = cntlin;
RUN;

FILENAME _src CATALOG 'work._xml';

DATA _NULL_;
  SET sasdata._xmlcolumn;
  IF _N_ = 1 THEN DO;
    FILE _src(_data.source);
    PUT 'DATA';
    FILE _src(_process.source);
    PUT '      ;';
    PUT '  SET sasdata._xml;';
    PUT '  WHERE _level IN (3 4 5);';
    PUT '  LENGTH var1-var256 $200 _type $16 _temp $200;';
    PUT '  ARRAY v {*} var1-var256;';
    PUT '  RETAIN var1-var256 " " _count . _repeat . _type ' ';';
    PUT '  IF _level = 3 AND _first = 1 THEN DO;';
    PUT '    _count = 0;';
    PUT '    _repeat = .;';
    PUT '    DO _i = 1 TO DIM(v);';
    PUT '      v(_i) = "";';
    PUT '    END;';
    PUT '  END;';
    PUT '  ELSE IF _level = 4 AND _first = 1 THEN DO;';
    PUT '    IF _repeat > 1 THEN DO _i = 1 TO (_repeat - 1);';
    PUT '      _count + 1;';
    PUT '      v(_count) = v(_count - 1);';
    PUT '    END;';
    PUT '    _type = "";';
    PUT '    _count + 1;';
    PUT '    _offset = INDEX(_text,
                             "table:number-columns-repeated=");';
    PUT '    IF _offset > 0 THEN _repeat = INPUT(SCAN(SUBSTR(_text,
               _offset + LENGTH("table:number-columns-repeated=") +
                                1), 1, """"), BEST.);';
    PUT '    ELSE _repeat = .;';
    PUT '    _offset = INDEX(_text, "office:value-type=");';
    PUT '    IF _offset > 0 THEN DO;';
    PUT '      _type = SCAN(SUBSTR(_text, _offset +
                 LENGTH("office:value-type=") + 1), 1, """");';
    PUT '      SELECT (_type);';
    PUT '        WHEN ("time") DO;';
    PUT '          _offset = INDEX(_text, "office:time-value=");';
    PUT '          _temp = SCAN(SUBSTR(_text, _offset +
                     LENGTH("office:time-value=") + 1), 1, """");';
    PUT '          v(_count) = PUT(HMS(INPUT(SCAN(_temp, 1, "PTHMS"),
                                       BEST.),';
    PUT '                              INPUT(SCAN(_temp, 2, "PTHMS"),
                                       BEST.),';
    PUT '                              INPUT(TRANSLATE(SCAN(_temp, 3,
                                             "PTHMS"), ".", ","),
                                             BEST.)), TIME11.2);';
    PUT '        END;';
    PUT '        WHEN ("date") DO;';
    PUT '          _offset = INDEX(_text, "office:date-value=");';
    PUT '          _temp = SCAN(SUBSTR(_text, _offset +
                     LENGTH("office:date-value=") + 1), 1, """");';
    PUT '          IF SCAN(_temp, 2, "T") NE " " THEN';
    PUT '            v(_count) = PUT(DHMS(INPUT(SCAN(_temp, 1, "T"),
                                     YYMMDD10.),';
    PUT '                   0, 0, INPUT(TRANSLATE(SCAN(_temp, 2, "T"),
                            ".", ","), TIME11.)), DATETIME21.2);';
    PUT '          ELSE';
    PUT '            v(_count) = PUT(INPUT(SCAN(_temp, 1, "T"),
                        YYMMDD10.), DATE9.);';
    PUT '        END;';
    PUT '        WHEN ("float") DO;';
    PUT '          _offset = INDEX(_text, "office:value=");';
    PUT '          v(_count) = SCAN(SUBSTR(_text, _offset +
                           LENGTH("office:value=") + 1), 1, """");';
    PUT '        END;';
    PUT '        WHEN ("percentage") DO;';
    PUT '          _offset = INDEX(_text, "office:value=");';
    PUT '          v(_count) = SCAN(SUBSTR(_text, _offset +
                           LENGTH("office:value=") + 1), 1, """");';
    PUT '        END;';
    PUT '        WHEN ("currency") DO;';
    PUT '          _offset = INDEX(_text, "office:currency=");';
    PUT '          v(_count) =
                     SCAN(SUBSTR(_text, _offset +
                       LENGTH("office:currency=") + 1), 1, """");';
    PUT '          _offset = INDEX(_text, "office:value=");';
    PUT '          v(_count) = STRIP(v(_count)) !!
                     SCAN(SUBSTR(_text, _offset +
                       LENGTH("office:value=") + 1), 1, """");';
    PUT '        END;';
    PUT '        OTHERWISE;';
    PUT '      END;';
    PUT '    END;';
    PUT '  END;';
    PUT '  ELSE IF _level = 5 AND _first NE 1 AND _last NE 1 THEN
         DO;';
    PUT '    IF NOT (_type IN ("time" "date" "float" "percentage"
                               "currency"))
               THEN v(_count) = STRIP(v(_count)) !! STRIP(_text);';
    PUT '  END;';
    FILE _src(_output.source);
    PUT '  IF _level = 3 AND _last = 1 THEN DO;';
    PUT '    IF _repeat > 1 THEN DO _i = 1 TO (_repeat - 1);';
    PUT '      _count + 1;';
    PUT '      v(_count) = v(_count - 1);';
    PUT '    END;';
    PUT '    SELECT (PUT(_record, rngtab.));';
    FILE _src(_run.source);
    PUT '      OTHERWISE;';
    PUT '    END;';
    PUT '  END;';
    PUT 'RUN;';
  END;
  j = STRIP(PUT(_columns, 16.));
  FILE _src(_data.source);
  PUT '    sasdata.' _table '(KEEP = var1-var' j +(-1) ')';
  FILE _src(_output.source);
  PUT '      WHEN ("' _table +(-1) '") OUTPUT sasdata.' _table ';';
RUN;

%INCLUDE _src(_data.source, _process.source, _output.source,
              _run.source) / SOURCE2;
