%GLOBAL _width;
%MACRO create_dsn(indsn=, outdsn=, max=);
  %LET _width = %LENGTH(&max.);
  %DO i = 1 %TO &max.;
    DATA &outdsn.%SYSFUNC(PUTN(&i., Z&_width..));
      SET &indsn.;
      version = &i.;
    RUN;
  %END;
%MEND;

%create_dsn(indsn=sashelp.cars, outdsn=cars_, max=10);

DATA cars_07 (DROP = _:);
  SET cars_07 (RENAME = (cylinders = _cylinders));
  cylinders = STRIP(PUT(_cylinders, BEST.));
RUN;

%MACRO to_char(first=, last=, var=);
  %DO _i = &first. %TO &last.;
    DATA cars_%SYSFUNC(PUTN(&_i., Z&_width..)) (DROP = _:);
      SET cars_%SYSFUNC(PUTN(&_i., Z&_width..))
           (RENAME = (&var. = _&var.));
      &var. = STRIP(PUT(_&var., BEST.));
    RUN;
  %END;
%MEND;

%to_char(first=3, last=8, var=horsepower);

DATA all_cars;
  SET cars_:;
RUN;

DATA set_list1 (KEEP = libname memname set_rename1);
  SET sashelp.vcolumn END = eof;
  WHERE UPCASE(libname) = 'WORK'
    AND UPCASE(memname) =: 'CARS_'
    AND UPCASE(name) = 'CYLINDERS'
        ;
  LENGTH set_rename1 $200;
  IF UPCASE(type) = 'CHAR' THEN
     set_rename1 = 'cylinders = _cylinders';
  OUTPUT;
RUN;

DATA set_list2 (KEEP = libname memname set_rename2);
  SET sashelp.vcolumn END = eof;
  WHERE UPCASE(libname) = 'WORK'
    AND UPCASE(memname) =: 'CARS_'
    AND UPCASE(name) = 'HORSEPOWER'
        ;
  LENGTH set_rename2 $200;
  IF UPCASE(type) = 'NUM' THEN
     set_rename2 = 'horsepower = _horsepower';
  OUTPUT;
RUN;

DATA set_list;
  MERGE set_list1
        set_list2
        ;
  BY libname memname;
  LENGTH set_name $200;
  set_name = STRIP(libname) || '.' || STRIP(memname);
  IF set_rename1 ne '' OR set_rename2 ne '' THEN DO;
    set_name = CATX(' '
                   ,set_name
                   ,'(RENAME = ('
                   ,set_rename1
                   ,set_rename2
                   ,'))'
                   );
  END;
RUN;

PROC SQL NOPRINT;
  SELECT STRIP(set_name)
  INTO   :set_list SEPARATED BY ' '
  FROM   set_list
  ;
QUIT;

DATA all_cars_corrected;
  SET &set_list.;
  IF _cylinders NE . THEN cylinders = INPUT(_cylinders, BEST.);
  IF _horsepower NE '' THEN horsepower = PUT(_horsepower, BEST.);
RUN;
